import React from "react";
import {
  Map,
  TileLayer,
  Marker,
  Popup,
  ZoomControl,
  Polyline
} from "react-leaflet";
import "./../css/maps.css";
import {calcDistance} from "./../helpers/distance";
import {longitude, latitude, mapWidth, mapHeight, testPath} from "./../config/variables";
import {Button, Input} from "reactstrap";
import {fetchPathKeys, fetchPaths, savePathToDb} from "./../helpers/dbHelpers.js"

export default class Leaflet extends React.Component{
  constructor(props){
    super(props);
    this.state = {
      latitude: latitude,
      longitude: longitude,
      zoom: 15,
      path: testPath,
      distance: 0,
      pathKey: "",
      pathKeys: [],
      savePathKey: "-"
    }

    this.savePathToDb = savePathToDb.bind(this)
  }

  componentDidMount(){
    this.getPathKeys()
  }

  getPathKeys = () => {
    fetchPathKeys().then(keys => this.setState({pathKeys: keys}))
  }

  getPath = () => {
    fetchPaths().then(paths => this.setState({path: paths[this.state.pathKey]}))
    this.getDistance()
  }

  choosePath = (event) => {
    this.setState({pathKey: event.target.value})
    this.getPath()
  }

  savePath = () => {
    this.savePathToDb().then(res => {
      let pathKeysCopy = this.state.pathKeys.slice();
      pathKeysCopy.push(this.state.savePathKey);
      this.setState({pathKeys: pathKeysCopy, savePathKey: ""})
    })
  }

  setSavePathKey = (event) => {
    this.setState({savePathKey: event.target.value})
  }

  addMarker = event => {
    const { lat, lng } = event.latlng
    let pathCopy = this.state.path.slice();
    pathCopy.push([lat, lng]);
    this.setState({path: pathCopy})
    this.getDistance()
  }

  updateMarker = (idx, event) => {
    let {lat, lng} = event.target.getLatLng();
    let pathCopy = this.state.path.slice();
    pathCopy[idx] = [lat, lng]
    this.setState({path: pathCopy})
    this.getDistance()
  }

  getDistance = () => {
    let dst = calcDistance(this.state.path)
    this.setState({distance: dst})
  }

  clearPath = () => {
    this.setState({path: [], distance: 0})
  }

  render() {
    const position = [this.state.latitude, this.state.longitude]
    return (
      <div>
        <div className="map">
          <Map
            center={position}
            zoom={this.state.zoom}
            zoomControl={false}
            style={{width: mapWidth, height: mapHeight}}
            onClick={this.addMarker}
          >
              <TileLayer
                attribution='&amp;copy <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
                url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
              />
              <ZoomControl position="bottomright" />

              {this.state.path.map((pos, idx) =>
                <Marker
                  position={pos}
                  draggable
                  key={idx}
                  onDragend={(e) => this.updateMarker(idx, e)}
                >
                  <Popup>
                    {idx}
                  </Popup>
                </Marker>
              )}

              <Polyline color="grey" positions={this.state.path} />
          </Map>
        </div>

        <div className="mapInfo">
          <Button onClick={this.clearPath}>Clear path</Button>
          <Button onClick={this.getDistance}>Calculate Distance</Button>

          <Input type="text" onChange={this.setSavePathKey} value={this.state.savePathKey}/>
          <Button onClick={this.savePath}>save path</Button>

          <Input type="select" name="select" id="savePath" onChange={this.choosePath.bind(this)}>
            {this.state.pathKeys.map((name, idx) =>
              <option value={name} key={idx}>{name}</option>
            )}
          </Input>

          <p>
            Distanz: {Math.round(this.state.distance*100)/100} km
          </p>
        </div>
      </div>
    );
  }
}